#include "antworld.h"
#include <QtGui>

AntWorld::AntWorld( QGraphicsScene * scene, QWidget * parent )
    :QGraphicsView(scene, parent)
{
    WORLDSIZE   = 1;
    STAGESIZE   = 1;
    homeX       = 0;
    homeY       = 0;
    foodX       = 0;
    foodY       = 0;
    showSPathTG = false;
    showSPathSF = true;
    showGrid    = true;
    showPheromoneTrails = false;
    //homeImage   = QImage(":/images/home.png");
    //foodImage   = QImage(":/images/food.gif");
}

void AntWorld::drawBackground(QPainter *painter, const QRectF &)//rect )
{
    QColor niceBlue(68, 132, 206);
    QPen bluePen(niceBlue, 1);
    QPen pathPen(QColor(0, 0, 0), 2);
    QPen redPen(QColor(255, 0, 0));
    QPen yellowPen(QColor(255, 128, 0, 255));
    QBrush pheromoneBrush(QColor(255, 168, 68, 255));
    redPen.setJoinStyle(Qt::RoundJoin);
    pathPen.setJoinStyle(Qt::RoundJoin);

    painter->setPen(bluePen);

    int unit = STAGESIZE / WORLDSIZE ;
    int step = 0;

    // draw grid
    if (showGrid)
        for (int i=0; i<=WORLDSIZE; i++)
        {
            // vertical (no leftovers)
            painter->drawLine(step, 0, step, unit*WORLDSIZE);
            // horizontal
            painter->drawLine(0,step, unit*WORLDSIZE, step);

            step += unit;
        }

    // draw pheromone trails
    if (showPheromoneTrails)
    {
        painter->setPen(yellowPen);
        int alpha;
        double max = pheromoneTrailsMatrix[0][0];
        for (int i=0; i<WORLDSIZE; i++)
            for (int j=0; j<WORLDSIZE; j++)
                if (max < pheromoneTrailsMatrix[i][j])
                    max = pheromoneTrailsMatrix[i][j];

        for (int i=0; i<WORLDSIZE; i++)
            for (int j=0; j<WORLDSIZE; j++)
            {
                alpha = (int)(pheromoneTrailsMatrix[i][j]/max*200);
                pheromoneBrush.setColor(QColor(255, 168, 68, alpha));
                painter->fillRect(i*unit+4, j*unit+4, unit-8, unit-8,
                                  pheromoneBrush);
            }
    }

    // draw path
    if (showSPathTG)
    {
        painter->setPen(redPen);
        painter->drawPath(shortestPathThisGen);
    }

    if (showSPathSF)
    {
        painter->setPen(pathPen);
        painter->drawPath(shortestPathSoFar);
    }

    painter->setPen(QPen(Qt::black));
    painter->setBrush(Qt::DiagCrossPattern);
    painter->drawPath(blockingPath);
    
    // draw home and food
    homeImage = QImage(":/images/home.png");
    foodImage = QImage(":/images/food.gif");
    
    if (homeImage.height() > unit)
    {
        homeImage = homeImage.scaled(homeImage.width()-(homeImage.width()-unit),
                homeImage.height()-(homeImage.height()-unit));
    }
    if (foodImage.height() > unit)
    {
        foodImage = foodImage.scaled(foodImage.width()-(foodImage.width()-unit),
                foodImage.height()-(foodImage.height()-unit));
    }

    painter->drawImage((homeX*unit+(unit/2))-(homeImage.width()/2),
                       (homeY*unit+(unit/2))-(homeImage.height()/2), homeImage);
    painter->drawImage((foodX*unit+(unit/2))-(foodImage.width()/2),
                       (foodY*unit+(unit/2))-(foodImage.height()/2), foodImage);
}

void AntWorld::setPheromoneTrails(double **matrix)
{
    pheromoneTrailsMatrix = matrix;
}

void AntWorld::setWorldSize(int const &ws)
{
    WORLDSIZE = ws;
}

int AntWorld::worldSize() const
{
    return WORLDSIZE;
}

void AntWorld::setStageSize(int const &ss)
{
    STAGESIZE = ss;
}

void AntWorld::setHome(int const &x, int const &y)
{
    homeX = x;
    homeY = y;
}

void AntWorld::setFood(int const &x, int const &y)
{
    foodX = x;
    foodY = y;
}

void AntWorld::setShortestPathThisGen(QPainterPath const &path)
{
    shortestPathThisGen = path;
}

void AntWorld::setShortestPathSoFar(QPainterPath const &path)
{
    shortestPathSoFar = path;
}

void AntWorld::showShortestPathThisGen(bool const &show)
{
    showSPathTG = show;
}

void AntWorld::showShortestPathSoFar(bool const &show)
{
    showSPathSF = show;
}

void AntWorld::setShowGrid(bool const &show)
{
    showGrid = show;
}

void AntWorld::setBlockingPath(QPainterPath const &path)
{
    blockingPath = path;
}

void AntWorld::showTrails(bool const &show)
{
    showPheromoneTrails = show;
}
