#include <QtGui>

#include "configdialog.h"

ConfigDialog::ConfigDialog(QWidget *parent):QDialog(parent)
{
    contentsWidget = new QListWidget;
    contentsWidget->setViewMode(QListView::IconMode);
    contentsWidget->setIconSize(QSize(48, 48));
    contentsWidget->setMovement(QListView::Static);
    contentsWidget->setMaximumWidth(80);
    contentsWidget->setMinimumWidth(80);
    
    antConfig = new AntConfig;
   // animConfig = new AnimConfig;
    
    pagesWidget = new QStackedWidget;
    pagesWidget->setMinimumWidth(450);
    pagesWidget->addWidget(antConfig);
    //pagesWidget->addWidget(animConfig);

    createIcons();
    contentsWidget->setCurrentRow(0);

    // buttons
    QPushButton *helpButton = new QPushButton(tr("&Help"));
    helpButton->setIcon(QIcon(":/images/dialog-help.png"));
    QPushButton *defaultsButton =  new QPushButton(tr("&Defaults"));
    defaultsButton->setIcon(QIcon(":/images/dialog-defaults.png"));
    okButton = new QPushButton(tr("Ok"));
    okButton->setDefault(true);
    okButton->setIcon(QIcon(":/images/dialog-ok.png"));
    cancelButton = new QPushButton(tr("Cancel"));
    cancelButton->setIcon(QIcon(":/images/dialog-cancel.png"));
    
    // make connextions
    connect(okButton, SIGNAL(clicked()),
	    this, SLOT(accept()));
    connect(cancelButton, SIGNAL(clicked()),
	    this, SLOT(reject()));    

    QHBoxLayout *horizontalLayout = new QHBoxLayout;
    horizontalLayout->addWidget(contentsWidget);
    horizontalLayout->addWidget(pagesWidget, 1);

    QHBoxLayout *buttonsLayout = new QHBoxLayout;
    buttonsLayout->addWidget(helpButton);
    buttonsLayout->addWidget(defaultsButton);
    buttonsLayout->addStretch(1);
    buttonsLayout->addWidget(okButton);
    buttonsLayout->addWidget(cancelButton);

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addLayout(horizontalLayout);
    mainLayout->addStretch(1);
    mainLayout->addSpacing(12);
    mainLayout->addLayout(buttonsLayout);
    setLayout(mainLayout);

    setWindowIcon(QIcon(":/images/config-dialog.png"));;
    setWindowTitle(tr("Configuration Dialog"));
}

void ConfigDialog::createIcons()
{
    QListWidgetItem *configButton = new QListWidgetItem(contentsWidget);
    configButton->setIcon(QIcon(":/images/dialog-ant.png"));
    configButton->setText(tr("Ants"));
    configButton->setTextAlignment(Qt::AlignHCenter);
    configButton->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
/*
    QListWidgetItem *animButton = new QListWidgetItem(contentsWidget);
    animButton->setIcon(QIcon(":/images/dialog-animation.png"));
    animButton->setText(tr("Animation"));
    //animButton->setTextAlignment(Qt::AlignVCenter);
    animButton->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
*/
    connect(contentsWidget,
            SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)),
            this, 
	    SLOT(changePage(QListWidgetItem*,QListWidgetItem*)));
}

void ConfigDialog::changePage(QListWidgetItem *current, QListWidgetItem *previous)
{
    if (!current)
        current = previous;

    pagesWidget->setCurrentIndex(contentsWidget->row(current));
}
