#include "mainwindow.h"
#include <QtGui>

MainWindow::MainWindow()
{
    setAttribute(Qt::WA_QuitOnClose);

    shortestPathStepsCount  = 32000;
    stageSize               = 600;
    WORLDSIZE               = 10;
    animationSpeed          = 2000;
    GENERATION              = 0;
    ANTS                    = 5;
    configDialog            = 0;
    cleanup                 = new QObjectCleanupHandler;
    antColony               = new AntCol(WORLDSIZE, ANTS);

    createActions();
    createMenus();
    createWidgets();
    readSettings();

    // generation #1
    nextGeneration();

    setWindowIcon(QIcon(":/images/icon.png"));
    setWindowTitle(tr("ANt"));
}

void MainWindow::removeGarbage()
{
    // clear the cleanup object
    cleanup->clear();
    // remove the items from the scene
    scene->clear();
}

void MainWindow::animateGeneration()
{
    // set animation speed
    animationSpeed = 4000/(speedSlider->value() + 1);
    // clear the previous animations
    removeGarbage();

    // create animations
    for (int i = 0; i < antColony->howManyAnts(); ++i)
    {
        Ant *ant = antColony->antsVector[i].gotFood ?
                   (new Ant(QColor(0,255,0), stageSize/WORLDSIZE/4)) :
                   (new Ant(QColor(0,0,255), stageSize/WORLDSIZE/4));
        ant->setPos(antColony->homeLocationX()*
                    (stageSize/WORLDSIZE)+(stageSize/WORLDSIZE/2),
                    antColony->homeLocationX()*
                    (stageSize/WORLDSIZE)+(stageSize/WORLDSIZE/2));
        scene->addItem(ant);

        QGraphicsItemAnimation *animation = new QGraphicsItemAnimation;
        animation->setItem(ant);

        // add for cleanup (*ant is removed in removeGarbage())
        cleanup->add(animation);

        // if ant has to get home -> double the time
        int factor = 1;
        if (antColony->antsVector[i].gotFood)
            factor = 2;

        // animation speed
        QTimeLine *timer =
              new QTimeLine(antColony->antsVector[i].positionInTabu*
                            (animationSpeed/WORLDSIZE)*factor);
        timer->setUpdateInterval(20);
        timer->setFrameRange(0, 100);
        cleanup->add(timer);

        qreal totalSteps = antColony->antsVector[i].positionInTabu;
        int antSteps = antColony->antsVector[i].positionInTabu;

        // set animation
        if (antColony->antsVector[i].gotFood)
        {
            totalSteps *= 20.0;
            for (int j=0; j<antSteps; j++)
            {
                // set animation at step (0,1,2... *10) / (20*steps)
                animation->setPosAt((j*10) / totalSteps,
                     QPointF(antColony->antsVector[i].tabuMatrix[0][j]*
                             (stageSize/WORLDSIZE)+(stageSize/WORLDSIZE/2),
                             antColony->antsVector[i].tabuMatrix[1][j]*
                             (stageSize/WORLDSIZE)+(stageSize/WORLDSIZE/2)));

                // set animation for return
                if (j != antSteps)
                    animation->setPosAt((totalSteps-(j*10)) / totalSteps,
                      QPointF(antColony->antsVector[i].tabuMatrix[0][j]*
                              (stageSize/WORLDSIZE)+(stageSize/WORLDSIZE/2),
                             antColony->antsVector[i].tabuMatrix[1][j]*
                             (stageSize/WORLDSIZE)+(stageSize/WORLDSIZE/2)));

                animation->setTimeLine(timer);
            }
        }
        // set animation --one way
        else
        {
            totalSteps *= 10.0;
            for (int j=0; j<antSteps; j++)
            {
                // set animation at step (0,1,2... *10) / (10*steps)
                animation->setPosAt((j*10) / totalSteps,
                     QPointF(antColony->antsVector[i].tabuMatrix[0][j]*
                             (stageSize/WORLDSIZE)+(stageSize/WORLDSIZE/2),
                             antColony->antsVector[i].tabuMatrix[1][j]*
                             (stageSize/WORLDSIZE)+(stageSize/WORLDSIZE/2)));
                animation->setTimeLine(timer);
            }
        }
        // start animation
        timer->start();
    }
}

void MainWindow::createActions()
{
    exitAction = new QAction(tr("E&xit"), this);
    exitAction->setIcon(QIcon(":/images/application-exit.png"));
    exitAction->setShortcut(tr("Ctrl+Q"));
    exitAction->setStatusTip(tr("Exit the application"));
    connect(exitAction, SIGNAL(triggered()), this, SLOT(close()));

    showConfigAction = new QAction(tr("&Configure Ant"), this);
    showConfigAction->setIcon(QIcon(":/images/config-dialog.png"));
    showConfigAction->setShortcut(tr("Ctrl+P"));
    showConfigAction->setStatusTip(tr("Configure the application"));
    connect(showConfigAction, SIGNAL(triggered()), this, SLOT(config()));

    aboutAction = new QAction(tr("About Tab"), this);
    aboutAction->setIcon(QIcon(":/images/help.png"));
    aboutAction->setStatusTip(tr("Show the Application's About box"));
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(about()));

    aboutQtAction = new QAction(tr("About &Qt"), this);
    aboutQtAction->setStatusTip(tr("Show the Qt library's About box"));
    connect(aboutQtAction, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
}

void MainWindow::createWidgets()
{
    // create the scene
    scene = new QGraphicsScene;
    scene->setSceneRect(-5, -5, stageSize+10, stageSize+10);
    scene->setItemIndexMethod(QGraphicsScene::NoIndex);

    // create the view
    view = new AntWorld(scene);
    view->setRenderHint(QPainter::Antialiasing);
    //view->setCacheMode(QGraphicsView::CacheBackground);
    view->setViewportUpdateMode(QGraphicsView::FullViewportUpdate);
    view->setDragMode(QGraphicsView::ScrollHandDrag);
    view->setWorldSize(WORLDSIZE);
    view->setStageSize(stageSize);
    view->setHome(antColony->homeLocationX(), antColony->homeLocationY());
    view->setFood(antColony->foodLocationX(), antColony->foodLocationY());
    view->setMinimumSize(stageSize, stageSize);
    view->resize(stageSize, stageSize);
    view->show();

    // initialize widgets
    startButton = new QPushButton(tr("Start Animation"));
    startButton->setIcon(QIcon(":/images/animation-start.png"));
    stopButton = new QPushButton(tr("Stop Animation"));
    stopButton->setIcon(QIcon(":/images/animation-stop.png"));
    nextGenButton = new QPushButton(tr("Advance "));
    nextGenButton->setIcon(QIcon(":/images/animation-seek-forward.png"));
    configButton = new QPushButton(tr("Configure"));
    configButton->setIcon(QIcon(":/images/config-dialog.png"));
    resetButton = new QPushButton(tr("Example"));

    QGroupBox *statusGroup = new QGroupBox("Status");
    QGridLayout *statusLayout = new QGridLayout;

    QLabel *genStaticLabel = new QLabel(tr("Generation: "));
    QLabel *antsStaticLabel = new QLabel(tr("Ants: "));
    QLabel *minStepsStaticLabel = new QLabel(tr("Steps: "));
    QLabel *worldSizeStaticLabel = new QLabel(tr("World Size: "));
    QLabel *speedLabel = new QLabel(tr("Animation Speed:"));

    speedSlider = new QSlider(Qt::Horizontal);
    speedSlider->setRange(0, 4);
    speedSlider->setValue(2);
    speedSlider->setMaximumWidth(200);;
    speedSlider->setTickPosition(QSlider::TicksBelow);
    generationsSpin = new QSpinBox;
    generationsSpin->setRange(1, 30);
    generationsSpin->setAlignment(Qt::AlignRight);
    generationsSpin->setMaximumWidth(50);

    showPathThisGen = new QCheckBox(tr("Shortest path this gen."));
    showPathSoFar = new QCheckBox(tr("Shortest path so far"));
    showTrailsCheckBox = new QCheckBox(tr("Show Pheromone Trails"));
    showGridCheckBox = new QCheckBox(tr("Show Grid"));
    showPathSoFar->setChecked(true);    
    showGridCheckBox->setChecked(true);

    genLabel = new QLabel(QString("%1").arg(GENERATION));
    antsLabel = new QLabel(QString("%1").arg(ANTS));
    minStepsLabel = new QLabel(QString("%1").arg(shortestPathStepsCount));
    worldSizeLabel = new QLabel(QString("%1x%2")
                                .arg(WORLDSIZE)
                                .arg(WORLDSIZE));

    statusLayout->addWidget(genStaticLabel, 0, 0);
    statusLayout->addWidget(genLabel, 0, 1, Qt::AlignRight);
    statusLayout->addWidget(antsStaticLabel, 1, 0);
    statusLayout->addWidget(antsLabel, 1, 1, Qt::AlignRight);
    statusLayout->addWidget(minStepsStaticLabel, 2, 0);
    statusLayout->addWidget(minStepsLabel, 2, 1, Qt::AlignRight);
    statusLayout->addWidget(worldSizeStaticLabel, 3, 0);
    statusLayout->addWidget(worldSizeLabel, 3, 1, Qt::AlignRight);

    statusGroup->setLayout(statusLayout);

    // make connections
    connect(startButton, SIGNAL(clicked()),
            this, SLOT(animateGeneration()));
    connect(stopButton, SIGNAL(clicked()),
            this, SLOT(removeGarbage()));
    connect(configButton, SIGNAL(clicked()),
            this, SLOT(config()));
    connect(nextGenButton, SIGNAL(clicked()),
            this, SLOT(nextGeneration()));
    connect(resetButton, SIGNAL(clicked()),
            this, SLOT(setDefaults()));

    QHBoxLayout *nextGenLayout = new QHBoxLayout;
    nextGenLayout->addWidget(nextGenButton);
    nextGenLayout->addWidget(generationsSpin);

    // layout the widgets
    QVBoxLayout *leftLayout = new QVBoxLayout;
    leftLayout->addWidget(startButton);
    leftLayout->addLayout(nextGenLayout);
    leftLayout->addWidget(stopButton);
    leftLayout->addWidget(statusGroup);
    leftLayout->addWidget(speedLabel);
    leftLayout->addWidget(speedSlider);
    leftLayout->addWidget(showPathThisGen);
    leftLayout->addWidget(showPathSoFar);
    leftLayout->addWidget(showTrailsCheckBox);
    leftLayout->addWidget(showGridCheckBox);
    leftLayout->addStretch(1);
    leftLayout->addWidget(resetButton);
    leftLayout->addWidget(configButton);

    QHBoxLayout *mainLayout = new QHBoxLayout;
    mainLayout->addLayout(leftLayout);
    mainLayout->addWidget(view);

    QWidget* centralWidget = new QWidget;
    centralWidget->setLayout(mainLayout);
    setCentralWidget(centralWidget);
}

void MainWindow::createMenus()
{
    fileMenu = menuBar()->addMenu(tr("&File"));
    fileMenu->addSeparator();
    fileMenu->addAction(exitAction);

    optionsMenu = menuBar()->addMenu(tr("&Options"));
    optionsMenu->addAction(showConfigAction);

    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAction);
    helpMenu->addAction(aboutQtAction);
}

void MainWindow::config()
{
    if (!configDialog)
    {
        configDialog = new ConfigDialog(this);
        configDialog->setModal(true);
        connect(configDialog, SIGNAL(accepted()),
                this, SLOT(setValues()));
    }

    configDialog->antConfig->homeLocationXEdit->
            setText(QString("%1").arg(antColony->homeLocationX()));
    configDialog->antConfig->homeLocationYEdit->
            setText(QString("%1").arg(antColony->homeLocationY()));
    configDialog->antConfig->foodLocationXEdit->
            setText(QString("%1").arg(antColony->foodLocationX()));
    configDialog->antConfig->foodLocationYEdit->
            setText(QString("%1").arg(antColony->foodLocationY()));
    configDialog->antConfig->pheromoneAlphaSpin->
            setValue(antColony->pheromoneAlpha());
    configDialog->antConfig->worldSizeSpin->
            setValue(antColony->worldSize());
    configDialog->antConfig->antsSpin->
            setValue(antColony->howManyAnts());
    configDialog->antConfig->heuristicBetaSpin->
            setValue(antColony->heuristicBeta());
    configDialog->antConfig->pheromoneEvapSpin->
            setValue(antColony->pheromoneEvap());

    configDialog->show();
    configDialog->raise();
    configDialog->activateWindow();
}

void MainWindow::about()
{
    QMessageBox::about(this, tr("About Ant"),
            tr("<h2>Ant 0.5.4</h2>"
               "<p>This is a front-end for libantcol.</p>"
               "<p>Copyright &copy; 2010 Petrescu Laurentiu</p>"
               "<p>petrescu.laur@gmail.com </p>"));
}

void MainWindow::nextGeneration()
{
    int min = 0;
    int ant = 0;
    
    
    QApplication::setOverrideCursor(Qt::WaitCursor);
    removeGarbage();
    for (int i=0; i<generationsSpin->value(); i++)
    {
        antColony->nextGen();
        GENERATION++;
        genLabel->setText(QString("%1").arg(GENERATION));
    
 
	// calculate shortest path
	min = 0;
	ant = 0;

	for (int i=1; i<antColony->howManyAnts(); i++)
	    if (antColony->antsVector[i].gotFood)
	    {
		ant = i;
		min = antColony->antsVector[i].positionInTabu;
		break;
	    }

	// search for minimum number of steps this generation
	for (int i=0; i<antColony->howManyAnts(); i++)
	    if (min > antColony->antsVector[i].positionInTabu &&
		antColony->antsVector[i].gotFood)
	    {
		min = antColony->antsVector[i].positionInTabu;
		ant = i;
	    }
	
	// erase old path
	QPainterPath path;
	
	path.moveTo(antColony->antsVector[ant].tabuMatrix[0][0]*
                    (stageSize/WORLDSIZE)+(stageSize/WORLDSIZE/2),
                antColony->antsVector[ant].tabuMatrix[1][0]*
                    (stageSize/WORLDSIZE)+(stageSize/WORLDSIZE/2));
	for (int i=1; i<min; i++)
	{
	    path.lineTo(antColony->antsVector[ant].tabuMatrix[0][i]*
                    (stageSize/WORLDSIZE)+(stageSize/WORLDSIZE/2),
                    antColony->antsVector[ant].tabuMatrix[1][i]*
                    (stageSize/WORLDSIZE)+(stageSize/WORLDSIZE/2));
	}

	if (shortestPathStepsCount > min && !path.isEmpty())
	{
	    shortestPathStepsCount = min;
	    shortestPathSoFar = path;	
	    minStepsLabel->setText(QString("%1")
				    .arg(shortestPathStepsCount));
	}
	
	view->setShortestPathThisGen(path);//
    }// end for
    
    view->setPheromoneTrails(antColony->pheromoneMatrix());
    view->setShortestPathSoFar(shortestPathSoFar);
    view->showShortestPathSoFar(showPathSoFar->isChecked());
    view->showShortestPathThisGen(showPathThisGen->isChecked());
    view->showTrails(showTrailsCheckBox->isChecked());
    view->setShowGrid(showGridCheckBox->isChecked());
    view->update();
    scene->update(0,0,stageSize,stageSize);

    QApplication::restoreOverrideCursor();
}

void MainWindow::setValues()
{
    // remove previous animations
    removeGarbage();    
    view->setBlockingPath(QPainterPath());

    // set parameters
    WORLDSIZE = configDialog->antConfig->worldSizeSpin->value();
    ANTS = configDialog->antConfig->antsSpin->value();
    antColony->setWorldSize(configDialog->antConfig->worldSizeSpin->value());
    antColony->setHowManyAnts(configDialog->antConfig->antsSpin->value());
    antColony->setFoodLocation(configDialog->antConfig->foodLocationXEdit->text().toInt(),
                               configDialog->antConfig->foodLocationYEdit->text().toInt());
    antColony->setHomeLocation(configDialog->antConfig->homeLocationXEdit->text().toInt(),
                               configDialog->antConfig->homeLocationYEdit->text().toInt());
    antColony->setPheromoneAlpha(configDialog->antConfig->pheromoneAlphaSpin->value());
    antColony->setHeuristicBeta(configDialog->antConfig->heuristicBetaSpin->value());
    antColony->setPheromoneEvap(configDialog->antConfig->pheromoneEvapSpin->value());

    // reinitialize the colony
    resetColony();
}

void MainWindow::resetColony()
{
    antColony->reinitialize();
    GENERATION = 0;
    shortestPathStepsCount = 32000;
    generationsSpin->setValue(1);
    nextGeneration();
    view->setWorldSize(WORLDSIZE);
    view->setHome(antColony->homeLocationX(), antColony->homeLocationY());
    view->setFood(antColony->foodLocationX(), antColony->foodLocationY());
    genLabel->setText(QString("%1")
                      .arg(GENERATION));
    antsLabel->setText(QString("%1")
                       .arg(ANTS));		      
    minStepsLabel->setText(QString("%1")
			.arg(shortestPathStepsCount));
    worldSizeLabel->setText(QString("%1x%2")
                                .arg(WORLDSIZE)
                                .arg(WORLDSIZE));

}

void MainWindow::writeSettings()
{
    QSettings settings("no.org", "Ant");

//    settings.setValue("state", saveState());
    settings.setValue("geometry", saveGeometry());

}

void MainWindow::readSettings()
{
    QSettings settings("no.org", "Ant");

//    restoreState(settings.value("state").toByteArray());
    restoreGeometry(settings.value("geometry").toByteArray());

}

void MainWindow::closeEvent(QCloseEvent */*event*/)
{
    writeSettings();
}

// this is the configuration for example
void MainWindow::setDefaults()
{
    removeGarbage();

    antColony->setWorldSize(8);
    WORLDSIZE = 8;
    antColony->setHowManyAnts(4);
    ANTS = 4;

    antColony->setHomeLocation(0, 4);
    antColony->setFoodLocation(7, 4);    
    antColony->setPheromoneAlpha(0);
    antColony->setHeuristicBeta(2);
    antColony->setPheromoneEvap(0.2);
    antColony->reinitialize();
    // top
    antColony->addToBlocked(1, 1);
    antColony->addToBlocked(2, 1);
    antColony->addToBlocked(3, 1);
    antColony->addToBlocked(4, 1);
    antColony->addToBlocked(5, 1);
    antColony->addToBlocked(6, 1);
    // middle
    antColony->addToBlocked(2, 3);
    antColony->addToBlocked(3, 3);
    antColony->addToBlocked(4, 3);
    antColony->addToBlocked(5, 3);
    antColony->addToBlocked(2, 4);
    antColony->addToBlocked(3, 4);
    antColony->addToBlocked(4, 4);
    antColony->addToBlocked(5, 4);
    // bottom
    antColony->addToBlocked(2, 6);
    antColony->addToBlocked(3, 6);
    antColony->addToBlocked(4, 6);
    antColony->addToBlocked(5, 6);
    antColony->addToBlocked(6, 6);
    antColony->addToBlocked(7, 6);

    QPainterPath blockingPath;
    // (1,1):(6,1)
    blockingPath.addRect(stageSize/WORLDSIZE, stageSize/WORLDSIZE,
                         stageSize/WORLDSIZE*6, stageSize/WORLDSIZE);
    // (2,3):(4,2)
    blockingPath.addRect(stageSize/WORLDSIZE*2, stageSize/WORLDSIZE*3,
                         stageSize/WORLDSIZE*4, stageSize/WORLDSIZE*2);
    // (2,6):(6,1)
    blockingPath.addRect(stageSize/WORLDSIZE*2, stageSize/WORLDSIZE*6,
                         stageSize/WORLDSIZE*6, stageSize/WORLDSIZE);


    view->setBlockingPath(blockingPath);
    GENERATION = 0;
    shortestPathStepsCount = 32000;
    generationsSpin->setValue(1);
    nextGeneration();
    view->setWorldSize(WORLDSIZE);
    view->setHome(antColony->homeLocationX(), antColony->homeLocationY());
    view->setFood(antColony->foodLocationX(), antColony->foodLocationY());
    genLabel->setText(QString("%1")
                      .arg(GENERATION));
    antsLabel->setText(QString("%1")
                       .arg(ANTS));
    minStepsLabel->setText(QString("%1")
		       .arg(shortestPathStepsCount));
    worldSizeLabel->setText(QString("%1x%2")
                                .arg(WORLDSIZE)
                                .arg(WORLDSIZE));

}

