#ifndef MAINWONDOW_H
#define MAINWINDOW_H

#include <QMainWindow> 
#include "libantcol.h"
#include "ant.h"
#include "antworld.h"
#include "configdialog.h"

class QObjectCleanupHandler;

class MainWindow: public QMainWindow
{
    Q_OBJECT
    
public:
    MainWindow();
    
private:
    // interface
    QPushButton *startButton;
    QPushButton *stopButton;
    QPushButton *configButton;
    QPushButton *nextGenButton;
    QPushButton *resetButton;
    QLabel *genLabel;
    QLabel *antsLabel;
    QLabel *minStepsLabel;
    QLabel *worldSizeLabel;
    QSlider *speedSlider;
    QSpinBox *generationsSpin;
    QCheckBox *showPathThisGen;
    QCheckBox *showPathSoFar;
    QCheckBox *showTrailsCheckBox;
    QCheckBox *showGridCheckBox;

    void createActions();
    void createMenus();
    void createWidgets();
    void readSettings();
    void writeSettings();

    QMenu *fileMenu;
    QMenu *optionsMenu;
    QMenu *helpMenu;

    QAction *exitAction;
    QAction *showConfigAction;
    QAction *aboutAction;
    QAction *aboutQtAction;

    ConfigDialog *configDialog;

    // animation
    QGraphicsScene *scene;
    QPainterPath shortestPathSoFar;
    AntWorld *view; // QGraphicsView
    AntCol *antColony;
    int shortestPathStepsCount;
    int GENERATION;
    int ANTS;
    int stageSize;
    int WORLDSIZE;
    int animationSpeed;


    // garbage collector
    QObjectCleanupHandler *cleanup;

private slots:
    void removeGarbage();
    void animateGeneration();
    void nextGeneration();
    void resetColony();
    void config();
    void setValues();
    void setDefaults();
    void about();

protected:
    void closeEvent(QCloseEvent *event);
};

#endif
