#include "pages.h"

AntConfig::AntConfig(QWidget *parent)
    : QWidget(parent)
{
    // validator for QLineEdit (0-99)
    QRegExp regExp("[0-9]{1,2}");
    
    // set world size
    QLabel *worldSizeLabel = new QLabel(tr("&World Size: "));
    worldSizeSpin = new QSpinBox;
    worldSizeSpin->setAlignment(Qt::AlignRight);
    worldSizeSpin->setRange(3, 30);
    worldSizeSpin->setValue(5); 
    worldSizeLabel->setBuddy(worldSizeSpin);
    
    // set how many ants
    QLabel *antsLabel = new QLabel(tr("How many &ants: "));
    antsSpin = new QSpinBox;
    antsSpin->setAlignment(Qt::AlignRight);
    antsSpin->setRange(1, 30);   
    antsSpin->setValue(3);
    antsLabel->setBuddy(antsSpin);

    // set food location X
    QLabel *foodLocationXLabel = new QLabel(tr("Food location (&X): "));
    foodLocationXEdit = new QLineEdit("2");
    foodLocationXEdit->setValidator(new QRegExpValidator(regExp, this));
    foodLocationXEdit->setAlignment(Qt::AlignRight);
    foodLocationXLabel->setBuddy(foodLocationXEdit);
    
    // set food location Y
    QLabel *foodLocationYLabel = new QLabel(tr("Food lo&cation (Y): "));
    foodLocationYEdit = new QLineEdit("2");
    foodLocationYEdit->setValidator(new QRegExpValidator(regExp, this));
    foodLocationYEdit->setAlignment(Qt::AlignRight);
    foodLocationYLabel->setBuddy(foodLocationYEdit);
    
    // set home location X
    QLabel *homeLocationXLabel = new QLabel(tr("Home &location (X): "));
    homeLocationXEdit = new QLineEdit("0");
    homeLocationXEdit->setValidator(new QRegExpValidator(regExp, this));
    homeLocationXEdit->setAlignment(Qt::AlignRight);
    homeLocationXLabel->setBuddy(homeLocationXEdit);
    
    // set home location Y
    QLabel *homeLocationYLabel = new QLabel(tr("Home l&ocation (Y): "));
    homeLocationYEdit = new QLineEdit("0");
    homeLocationYEdit->setValidator(new QRegExpValidator(regExp, this));
    homeLocationYEdit->setAlignment(Qt::AlignRight);
    homeLocationYLabel->setBuddy(homeLocationYEdit);
    
    // set pheromone factor
    QLabel *pheromoneAlphaLabel = new QLabel(tr("Pheromone factor (alpha): "));
    pheromoneAlphaSpin = new QSpinBox;
    pheromoneAlphaSpin->setAlignment(Qt::AlignRight);
    pheromoneAlphaSpin->setRange(0, 10);
    pheromoneAlphaLabel->setBuddy(pheromoneAlphaSpin);
    
    // set heuristic factor
    QLabel *heuristicBetaLabel = new QLabel(tr("Heuristic factor (beta): "));
    heuristicBetaSpin = new QSpinBox;
    heuristicBetaSpin->setAlignment(Qt::AlignRight);
    heuristicBetaSpin->setRange(0, 10); 
    heuristicBetaLabel->setBuddy(heuristicBetaSpin);
    
    // set evaporation rate
    QLabel *pheromoneEvapLabel = new QLabel(tr("Pheromone evaporation rate (ro): "));
    pheromoneEvapSpin = new QDoubleSpinBox;
    pheromoneEvapSpin->setAlignment(Qt::AlignRight);
    pheromoneEvapSpin->setDecimals(2); 
    pheromoneEvapSpin->setRange(0.0, 0.99); 
    pheromoneEvapSpin->setSingleStep(0.05);
    pheromoneEvapLabel->setBuddy(pheromoneEvapSpin);
    
    QGroupBox *groupBox = new QGroupBox(tr("Simulation parameters"));
    
    QGridLayout *topLayout = new QGridLayout;
    topLayout->addWidget(worldSizeLabel, 	0, 0);
    topLayout->addWidget(worldSizeSpin, 	0, 1);
    topLayout->addWidget(antsLabel,	 	1, 0);
    topLayout->addWidget(antsSpin,	 	1, 1);
    topLayout->addWidget(foodLocationXLabel,	2, 0);
    topLayout->addWidget(foodLocationXEdit, 	2, 1);
    topLayout->addWidget(foodLocationYLabel, 	3, 0);
    topLayout->addWidget(foodLocationYEdit, 	3, 1);
    topLayout->addWidget(homeLocationXLabel, 	4, 0);
    topLayout->addWidget(homeLocationXEdit, 	4, 1);
    topLayout->addWidget(homeLocationYLabel, 	5, 0);
    topLayout->addWidget(homeLocationYEdit, 	5, 1);
    topLayout->addWidget(pheromoneAlphaLabel, 	6, 0);
    topLayout->addWidget(pheromoneAlphaSpin, 	6, 1);
    topLayout->addWidget(heuristicBetaLabel, 	7, 0);
    topLayout->addWidget(heuristicBetaSpin, 	7, 1);
    topLayout->addWidget(pheromoneEvapLabel, 	8, 0);
    topLayout->addWidget(pheromoneEvapSpin, 	8, 1);
    
    groupBox->setLayout(topLayout);    
    
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(groupBox);
    mainLayout->addStretch(1);
    
    setLayout(mainLayout);
}

AnimConfig::AnimConfig(QWidget *parent)
     : QWidget(parent)
{
    QLabel *label = new QLabel(tr("Not Implemented"));
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(label);

    setLayout(mainLayout);
}
